@@ -4,30 +4,28 @@ from pathlib import Path
 import arrow
 import click
 import pyperclip
-from typing import cast
 
 from simpletodo.model import (
     ErrMsg,
     Repeat,
-    TodoItem,
     TodoStatus,
     new_todoitem,
     now,
 )
 from simpletodo.util import (
     ensure_db_file,
+    load_cfg,
     load_db,
     print_donelist,
     print_repeatlist,
     print_result,
     print_todolist,
     split_lists,
-    stop_schedule,
     update_db,
+    upgrade_to_v016,
     validate_n,
     make_schedule,
     update_schedules,
-    load_cfg,
     change_db_path,
     todo_cfg_path,
     print_mottos,
@@ -108,12 +106,13 @@ def cli(ctx, show_all, new_path):
     https://pypi.org/project/simpletodo/
     """
     if ctx.invoked_subcommand is None:
+        cfg = load_cfg()
         if new_path:
-            err = change_db_path(Path(new_path))
+            err = change_db_path(Path(new_path), cfg)
             check(ctx, err)
             ctx.exit()
 
-        db = load_db()
+        db = load_db(cfg)
 
         # 显示格言
         if (not db["hide_motto"]) and db["mottos"]:
@@ -128,7 +127,7 @@ def cli(ctx, show_all, new_path):
                     click.echo(f"\n【{random.choice(db['mottos'])}】")
 
         # 显示 todo
-        update_schedules(db)
+        update_schedules(db, cfg)
         if not db["items"]:
             click.echo("There's no todo item.")
             click.echo("Use 'todo add ...' to add a todo item.")
@@ -160,15 +159,15 @@ def add(ctx, event):
 
     Example: todo add Buy more beer.
     """
-    event = cast(tuple[str], event)
     subject = " ".join(event).strip()
     if not subject:
         click.echo(ctx.get_help())
         ctx.exit()
 
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     db["items"].insert(0, new_todoitem(subject))
-    update_db(db)
+    update_db(db, cfg)
     print_result(db)
     ctx.exit()
 
@@ -183,12 +182,16 @@ def copy(ctx, n):
 
     Example: todo copy 3
     """
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     err = validate_n(db["items"], n)
     check(ctx, err)
 
     content = db["items"][n - 1]["event"]
-    pyperclip.copy(content)
+    try:
+        pyperclip.copy(content)
+    except Exception:
+        pass
     ctx.exit()
 
 
@@ -200,20 +203,25 @@ def done(ctx, n):
 
     Example: todo done 1
     """
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     err = validate_n(db["items"], n)
     check(ctx, err)
 
     idx = n - 1
     status = db["items"][idx]["status"]
-    every = db["items"][idx]["repeat"]
-    if TodoStatus[status] is TodoStatus.Completed:
-        click.echo("Warning: It was in the completed-list, nothing changes.")
-    db["items"][idx]["status"] = TodoStatus.Completed.name
-    if Repeat[every] is Repeat.Never:
-        # 如果设置了重复提醒，则不修改 dtime (让它的值保持为零)
+    repeat = db["items"][idx]["repeat"]
+    if TodoStatus[status] is not TodoStatus.Incomplete:
+        click.echo("Warning: It is not in the incomplete-list, nothing changes.")
+        ctx.exit()
+
+    if Repeat[repeat] is Repeat.Never:
         db["items"][idx]["dtime"] = now()
-    update_db(db)
+        db["items"][idx]["status"] = TodoStatus.Completed.name
+    else:
+        db["items"][idx]["status"] = TodoStatus.Waiting.name
+
+    update_db(db, cfg)
     ctx.exit()
 
 
@@ -225,11 +233,12 @@ def delete(ctx, n):
 
     Example: todo delete 2
     """
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     err = validate_n(db["items"], n)
     check(ctx, err)
     del db["items"][n - 1]
-    update_db(db)
+    update_db(db, cfg)
     print_result(db)
     ctx.exit()
 
@@ -238,13 +247,13 @@ def delete(ctx, n):
 @click.pass_context
 def clean(ctx):
     """Clear the completed list (delete all completed items)."""
-    db = load_db()
-    head, _, tail = split_lists(db)
-    items: list[TodoItem] = []
-    for _, item in head+tail:
-        items.append(item)
+    cfg = load_cfg()
+    db = load_db(cfg)
+    items = [
+        x for x in db["items"] if TodoStatus[x["status"]] is not TodoStatus.Completed
+    ]
     db["items"] = items
-    update_db(db)
+    update_db(db, cfg)
     print_result(db)
     ctx.exit()
 
@@ -257,19 +266,21 @@ def redo(ctx, n):
 
     Example: todo redo 1
     """
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     err = validate_n(db["items"], n)
     check(ctx, err)
 
     idx = n - 1
     status = db["items"][idx]["status"]
-    if TodoStatus[status] is TodoStatus.Incomplete:
-        click.echo("Warning: It is in the incomplete-list, nothing changes.")
+    if TodoStatus[status] is not TodoStatus.Completed:
+        click.echo("Warning: It is not in the completed-list, nothing changes.")
+        ctx.exit()
 
     db["items"][idx]["status"] = TodoStatus.Incomplete.name
     db["items"][idx]["ctime"] = now()
     db["items"][idx]["dtime"] = 0
-    update_db(db)
+    update_db(db, cfg)
     ctx.exit()
 
 
@@ -282,30 +293,17 @@ def redo(ctx, n):
     "--start-from",
     help="Example: -from 2021-04-01",
 )
-@click.option(
-    "stop",
-    "-stop",
-    "--stop",
-    is_flag=True,
-    help="Stop repeating the event (删除指定事项的周期计划)",
-)
 @click.pass_context
-def repeat(ctx, n, every, start: str, stop):
+def repeat(ctx, n, every, start: str):
     """Set the N'th item to repeat every week/month/year.
 
     Example: todo repeat 1 -every month -from today
     """
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     err = validate_n(db["items"], n)
     check(ctx, err)
 
-    idx = n - 1
-    item = db["items"][idx]
-
-    # 优先处理 stop
-    if stop:
-        stop_schedule(db, idx, item, ctx)
-
     # 为了逻辑清晰，要求同时设置重复模式与起始时间
     if not every:
         click.echo("Error: Missing option '-every'.")
@@ -325,8 +323,9 @@ def repeat(ctx, n, every, start: str, stop):
         case _:
             s_date = arrow.get(start)
 
+    idx = n - 1
     make_schedule(db, idx, every, s_date, ctx)
-    update_db(db)
+    update_db(db, cfg)
     ctx.exit()
 
 
@@ -341,7 +340,8 @@ def edit(ctx, args):
     Example: todo edit 1 "Meet John on friday."
     """
     n, subject = args
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     err = validate_n(db["items"], n)
     check(ctx, err)
 
@@ -351,7 +351,7 @@ def edit(ctx, args):
         ctx.exit()
 
     db["items"][n - 1]["event"] = subject
-    update_db(db)
+    update_db(db, cfg)
     ctx.exit()
 
 
@@ -376,14 +376,26 @@ def edit(ctx, args):
 @click.option("edit", "-e", "--edit", type=(int, str), help="Edit a motto.")
 @click.option("del_n", "-d", "--delete", type=int, help="Example: todo motto -d 1")
 @click.pass_context
-def motto(ctx, show_list, is_show, is_hide, randomly, select, top, sentence, edit:tuple[int, str], del_n):
+def motto(
+    ctx,
+    show_list,
+    is_show,
+    is_hide,
+    randomly,
+    select,
+    top,
+    sentence,
+    edit: tuple[int, str],
+    del_n,
+):
     """Motto (格言/座右铭/目标)
-    
+
     Control how to display a motto when listing todo items.
 
     设置格言，可显示也可隐藏，如果设为显示，则会在待办事项列表的上方显示。
     """
-    db = load_db()
+    cfg = load_cfg()
+    db = load_db(cfg)
     mottos = db["mottos"]
     hide_motto = db["hide_motto"]
     select_n = db["select_motto"]
@@ -394,12 +406,12 @@ def motto(ctx, show_list, is_show, is_hide, randomly, select, top, sentence, edi
 
     if is_show:
         db["hide_motto"] = False
-        update_db(db)
+        update_db(db, cfg)
         ctx.exit()
 
     if is_hide:
         db["hide_motto"] = True
-        update_db(db)
+        update_db(db, cfg)
         ctx.exit()
 
     if sentence:
@@ -408,27 +420,27 @@ def motto(ctx, show_list, is_show, is_hide, randomly, select, top, sentence, edi
             click.echo(ctx.get_help())
             ctx.exit()
         db["mottos"].append(sentence)
-        update_db(db)
+        update_db(db, cfg)
         ctx.exit()
 
     if edit:
         n, value = edit
         err = validate_n(db["mottos"], n)
         check(ctx, err)
-        db["mottos"][n-1] = value
-        update_db(db)
+        db["mottos"][n - 1] = value
+        update_db(db, cfg)
         ctx.exit()
 
     if randomly:
         db["select_motto"] = 0
-        update_db(db)
+        update_db(db, cfg)
         ctx.exit()
 
     if select:
         err = validate_n(db["mottos"], select)
         check(ctx, err)
         db["select_motto"] = select
-        update_db(db)
+        update_db(db, cfg)
         ctx.exit()
 
     if top:
@@ -436,7 +448,7 @@ def motto(ctx, show_list, is_show, is_hide, randomly, select, top, sentence, edi
         check(ctx, err)
         item = db["mottos"].pop(top - 1)
         db["mottos"].insert(0, item)
-        update_db(db)
+        update_db(db, cfg)
         print_mottos(mottos, hide_motto, select_n)
         ctx.exit()
 
@@ -444,7 +456,7 @@ def motto(ctx, show_list, is_show, is_hide, randomly, select, top, sentence, edi
         err = validate_n(db["mottos"], del_n)
         check(ctx, err)
         del db["mottos"][del_n - 1]
-        update_db(db)
+        update_db(db, cfg)
         print_mottos(mottos, hide_motto, select_n)
         ctx.exit()
 
@@ -453,8 +465,9 @@ def motto(ctx, show_list, is_show, is_hide, randomly, select, top, sentence, edi
 
 
 # 初始化
-ensure_db_file()
-db_path = load_cfg()["db_path"]
+cfg = ensure_db_file()
+db_path = cfg["db_path"]
+upgrade_to_v016()
 
 if __name__ == "__main__":
     cli(obj={})
