@@ -5,7 +5,6 @@ import click
 import json
 import arrow
 from pathlib import Path
-from typing import cast
 from appdirs import AppDirs
 from arrow.arrow import Arrow
 
@@ -14,12 +13,11 @@ from simpletodo.model import (
     ErrMsg,
     IdxTodoList,
     Repeat,
-    TodoItem,
     TodoStatus,
     new_db,
-    now,
     TodoConfig,
 )
+from . import __version__
 
 todo_cfg_name = "todo-config.json"
 todo_db_name = "todo-db.json"
@@ -31,26 +29,30 @@ todo_cfg_path = app_config_dir.joinpath(todo_cfg_name)
 default_db_path = app_config_dir.joinpath(todo_db_name)
 
 
+def write_cfg(cfg: TodoConfig) -> None:
+    with open(todo_cfg_path, "w", encoding="utf-8") as f:
+        json.dump(cfg, f, indent=4, ensure_ascii=False)
+
+
 def ensure_cfg_file() -> None:
     app_config_dir.mkdir(parents=True, exist_ok=True)
     if not todo_cfg_path.exists():
-        with open(todo_cfg_path, "w", encoding="utf-8") as f:
-            cfg = TodoConfig(db_path=default_db_path.__str__())
-            json.dump(cfg, f, indent=4, ensure_ascii=False)
+        cfg = TodoConfig(db_path=default_db_path.__str__(), upgrade="")
+        write_cfg(cfg)
 
 
-def ensure_db_file() -> None:
+def ensure_db_file() -> TodoConfig:
     ensure_cfg_file()
     cfg = load_cfg()
     db_path = Path(cfg["db_path"])
     if not db_path.exists():
         with open(db_path, "w", encoding="utf-8") as f:
             json.dump(new_db(), f, indent=4, ensure_ascii=False)
+    return cfg
 
 
-def change_db_path(new_path: Path) -> ErrMsg:
+def change_db_path(new_path: Path, cfg: TodoConfig) -> ErrMsg:
     """new_path 是一个不存在的文件或一个已存在的文件夹，不能是一个已存在的文件"""
-    cfg = load_cfg()
     new_path = new_path.resolve()
     if new_path.is_dir():
         new_path = new_path.joinpath(todo_db_name)
@@ -67,11 +69,14 @@ def change_db_path(new_path: Path) -> ErrMsg:
 
 def load_cfg() -> TodoConfig:
     with open(todo_cfg_path, "rb") as f:
-        return cast(TodoConfig, json.load(f))
+        cfg_dict = json.load(f)
+        return TodoConfig(
+            db_path=cfg_dict["db_path"],
+            upgrade=cfg_dict.get("upgrade", ""),
+        )
 
 
-def load_db() -> DB:
-    cfg = load_cfg()
+def load_db(cfg: TodoConfig) -> DB:
     with open(cfg["db_path"], "rb") as f:
         db_dict = json.load(f)
         return DB(
@@ -89,15 +94,15 @@ def split_lists(db: DB) -> tuple[IdxTodoList, IdxTodoList, IdxTodoList]:
     repeat_list: IdxTodoList = []
 
     for idx, item in enumerate(db["items"]):
-        # 只要 status 是 Incomplete 就归入待办列表
-        if TodoStatus[item["status"]] is TodoStatus.Incomplete:
-            todo_list.append((idx, item))
-        # 只要完成时间大于零就归入已完成列表
-        if item["dtime"] > 0:
-            done_list.append((idx, item))
-        # 只要 repeat 不是 Never 就归入计划任务列表
-        if Repeat[item["repeat"]] is not Repeat.Never:
-            repeat_list.append((idx, item))
+        match TodoStatus[item["status"]]:
+            case TodoStatus.Incomplete:
+                todo_list.append((idx, item))
+            case TodoStatus.Completed:
+                done_list.append((idx, item))
+            case TodoStatus.Waiting:
+                repeat_list.append((idx, item))
+            case _:
+                raise ValueError(f"Unknown status: {item['status']}")
 
     todo_list.sort(key=lambda x: x[1]["ctime"], reverse=True)
     done_list.sort(key=lambda x: x[1]["dtime"], reverse=True)
@@ -105,8 +110,7 @@ def split_lists(db: DB) -> tuple[IdxTodoList, IdxTodoList, IdxTodoList]:
     return todo_list, done_list, repeat_list
 
 
-def update_db(db: DB) -> None:
-    cfg = load_cfg()
+def update_db(db: DB, cfg: TodoConfig) -> None:
     with open(cfg["db_path"], "w", encoding="utf-8") as f:
         json.dump(db, f, indent=4, ensure_ascii=False)
 
@@ -186,25 +190,6 @@ def validate_n(a_list: list, n: int) -> ErrMsg:
     return ""
 
 
-def stop_schedule(
-    db: DB,
-    idx: int,
-    item: TodoItem,
-    ctx: click.Context,
-) -> None:
-    if Repeat[item["repeat"]] is Repeat.Never:
-        click.echo("Warning: It is not set to repeat, nothing changes.")
-        ctx.exit()
-    db["items"][idx]["repeat"] = Repeat.Never.name
-    db["items"][idx]["s_date"] = ""
-    db["items"][idx]["n_date"] = ""
-    if TodoStatus[item["status"]] is TodoStatus.Completed:
-        # 只有当该事项状态为 Completed 时，才需要设置 dtime
-        db["items"][idx]["dtime"] = now()
-    update_db(db)
-    ctx.exit()
-
-
 def make_schedule(db: DB, i: int, every: str, start: Arrow, ctx: click.Context) -> None:
     """Set up a new schedule (repeat event)."""
 
@@ -219,8 +204,7 @@ def make_schedule(db: DB, i: int, every: str, start: Arrow, ctx: click.Context)
 
     # set "dtime"
     # 一个事件只要设置了重复提醒，那么它的 dtime 就必须为零
-    if db["items"][i]["dtime"]:
-        db["items"][i]["dtime"] = 0
+    db["items"][i]["dtime"] = 0
 
     # set "repeat"
     repeat = every.capitalize()
@@ -233,43 +217,71 @@ def make_schedule(db: DB, i: int, every: str, start: Arrow, ctx: click.Context)
     # set "status" and "n_date"
     # 在本函数的开头已经验证过 start, 防止其小于今天。
     if start > today.ceil("day"):  # ceil("day") 返回本地时间当天最后一秒
-        db["items"][i]["status"] = TodoStatus.Completed.name
+        db["items"][i]["status"] = TodoStatus.Waiting.name
         db["items"][i]["n_date"] = db["items"][i]["s_date"]
     if start.format(DateFormat) == today.format(DateFormat):
         db["items"][i]["status"] = TodoStatus.Incomplete.name
         n_date = shift_next_date(start, start, Repeat[repeat])
-        db["items"][i]["n_date"] = n_date.format(DateFormat)
+        db["items"][i]["n_date"] = n_date
 
 
-# 在调用本函数之前，要限定 repeat 的值的范围。
-def shift_next_date(s_date: Arrow, n_date: Arrow, repeat: Repeat) -> Arrow:
-    match repeat:
-        case Repeat.Week:
-            return n_date.shift(days=7)
-        case Repeat.Month:
-            if is_last_day(s_date):
-                return n_date.shift(months=1).ceil("month")
-            return n_date.shift(months=1)
-        case Repeat.Year:
-            if is_last_day(s_date):
-                return n_date.shift(year=1).ceil("month")
-            return n_date.shift(year=1)
-        case _:
-            raise ValueError(repeat)
+def shift_next_date(s_date: Arrow, n_date: Arrow, repeat: Repeat) -> str:
+    today = arrow.now().format(DateFormat)
+    next_date = n_date.format(DateFormat)
+    while next_date <= today:
+        match repeat:
+            case Repeat.Week:
+                n_date = n_date.shift(days=7)
+            case Repeat.Month:
+                if is_last_day(s_date):
+                    n_date = n_date.shift(months=1).ceil("month")
+                else:
+                    n_date = n_date.shift(months=1)
+            case Repeat.Year:
+                if is_last_day(s_date):
+                    n_date = n_date.shift(year=1).ceil("month")
+                else:
+                    n_date = n_date.shift(year=1)
+            case _:
+                raise ValueError(repeat)
+
+        next_date = n_date.format(DateFormat)
+
+    return next_date
 
 
-def update_schedules(db: DB) -> None:
-    today = arrow.now()
+def update_schedules(db: DB, cfg: TodoConfig, force: bool = False) -> None:
+    today = arrow.now().format(DateFormat)
     u_date = today.format(DateFormat)
-    if u_date == db["u_date"]:
+    if not force and u_date == db["u_date"]:
         # 如果今天已经更新过，就不用更新了（每天只更新一次）
         return
     db["u_date"] = u_date
     for idx, item in enumerate(db["items"]):
-        if today.format(DateFormat) == item["n_date"]:
+        if TodoStatus[item["status"]] is TodoStatus.Waiting and today >= item["n_date"]:
             db["items"][idx]["status"] = TodoStatus.Incomplete.name
             s_date = arrow.get(item["s_date"])
             n_date = arrow.get(item["n_date"])
-            n_date = shift_next_date(s_date, n_date, Repeat[item["repeat"]])
-            db["items"][idx]["n_date"] = n_date.format(DateFormat)
-    update_db(db)
+            next_date = shift_next_date(s_date, n_date, Repeat[item["repeat"]])
+            db["items"][idx]["n_date"] = next_date
+    update_db(db, cfg)
+
+
+def upgrade_to_v016() -> None:
+    """Upgrade to v0.1.6
+
+    从低于 v0.1.6 升级到 v0.1.6 及以上时自动升级。
+    """
+    cfg = load_cfg()
+    if cfg["upgrade"] == "0.1.6" or __version__ < "0.1.6":
+        return
+
+    print("Upgrading to v0.1.6...")
+    cfg["upgrade"] = "0.1.6"
+    write_cfg(cfg)
+
+    db = load_db(cfg)
+    for idx, item in enumerate(db["items"]):
+        if TodoStatus[item["status"]] is TodoStatus.Completed and item["dtime"] <= 0:
+            db["items"][idx]["status"] = TodoStatus.Waiting.name
+    update_schedules(db, cfg, force=True)
